@@ -3,8 +3,8 @@
 Miscellaneous
 =============
 
-IEEE 754 Floating Point Special Values:
------------------------------------------------
+IEEE 754 Floating Point Special Values
+--------------------------------------
 
 Special values defined in numpy: nan, inf,
 
@@ -13,12 +13,11 @@ original value was)
 
 Note: cannot use equality to test NaNs. E.g.: ::
 
- >>> np.where(myarr == np.nan)
- >>> nan == nan  # is always False! Use special numpy functions instead.
-
- >>> np.nan == np.nan
- False
  >>> myarr = np.array([1., 0., np.nan, 3.])
+ >>> np.nonzero(myarr == np.nan)
+ (array([], dtype=int64),)
+ >>> np.nan == np.nan  # is always False! Use special numpy functions instead.
+ False
  >>> myarr[myarr == np.nan] = 0. # doesn't work
  >>> myarr
  array([  1.,   0.,  NaN,   3.])
@@ -32,8 +31,8 @@ Other related special value functions: ::
  isfinite(): True if not nan or inf
  nan_to_num(): Map nan to 0, inf to max float, -inf to min float
 
-The following corresponds to the usual functions except that nans are excluded from
-the results: ::
+The following corresponds to the usual functions except that nans are excluded
+from the results: ::
 
  nansum()
  nanmax()
@@ -49,29 +48,33 @@ the results: ::
  42.0
 
 How numpy handles numerical exceptions
+--------------------------------------
 
-Default is to "warn"
-But this can be changed, and it can be set individually for different kinds
-of exceptions. The different behaviors are: ::
+The default is to ``'warn'`` for ``invalid``, ``divide``, and ``overflow``
+and ``'ignore'`` for ``underflow``.  But this can be changed, and it can be
+set individually for different kinds of exceptions. The different behaviors
+are:
 
- 'ignore' : ignore completely
- 'warn'   : print a warning (once only)
- 'raise'  : raise an exception
- 'call'   : call a user-supplied function (set using seterrcall())
+ - 'ignore' : Take no action when the exception occurs.
+ - 'warn'   : Print a `RuntimeWarning` (via the Python `warnings` module).
+ - 'raise'  : Raise a `FloatingPointError`.
+ - 'call'   : Call a function specified using the `seterrcall` function.
+ - 'print'  : Print a warning directly to ``stdout``.
+ - 'log'    : Record error in a Log object specified by `seterrcall`.
 
-These behaviors can be set for all kinds of errors or specific ones: ::
+These behaviors can be set for all kinds of errors or specific ones:
 
- all:       apply to all numeric exceptions
- invalid:   when NaNs are generated
- divide:    divide by zero (for integers as well!)
- overflow:  floating point overflows
- underflow: floating point underflows
+ - all       : apply to all numeric exceptions
+ - invalid   : when NaNs are generated
+ - divide    : divide by zero (for integers as well!)
+ - overflow  : floating point overflows
+ - underflow : floating point underflows
 
 Note that integer divide-by-zero is handled by the same machinery.
-These behaviors are set on a per-thead basis.
+These behaviors are set on a per-thread basis.
 
-Examples:
-------------
+Examples
+--------
 
 ::
 
@@ -84,17 +87,18 @@ Examples:
  >>> np.sqrt(np.array([-1.]))
  FloatingPointError: invalid value encountered in sqrt
  >>> def errorhandler(errstr, errflag):
- ...      print "saw stupid error!"
+ ...      print("saw stupid error!")
  >>> np.seterrcall(errorhandler)
+ <function err_handler at 0x...>
  >>> j = np.seterr(all='call')
  >>> np.zeros(5, dtype=np.int32)/0
  FloatingPointError: invalid value encountered in divide
  saw stupid error!
  >>> j = np.seterr(**oldsettings) # restore previous
-                                  # error-handling settings
+ ...                              # error-handling settings
 
-Interfacing to C:
------------------
+Interfacing to C
+----------------
 Only a survey of the choices. Little detail on how each works.
 
 1) Bare metal, wrap your own C-code manually.
@@ -118,31 +122,24 @@ Only a survey of the choices. Little detail on how each works.
 
    - API will change for Python 3.0!
 
-2) pyrex
+2) Cython
 
  - Plusses:
 
    - avoid learning C API's
    - no dealing with reference counting
-   - can code in psuedo python and generate C code
+   - can code in pseudo python and generate C code
    - can also interface to existing C code
    - should shield you from changes to Python C api
-   - become pretty popular within Python community
+   - has become the de-facto standard within the scientific Python community
+   - fast indexing support for arrays
 
  - Minuses:
 
    - Can write code in non-standard form which may become obsolete
    - Not as flexible as manual wrapping
-   - Maintainers not easily adaptable to new features
 
-Thus:
-
-3) cython - fork of pyrex to allow needed features for SAGE
-
-  - being considered as the standard scipy/numpy wrapping tool
-  - fast indexing support for arrays
-
-4) ctypes
+3) ctypes
 
  - Plusses:
 
@@ -161,9 +158,10 @@ Thus:
 
  - Minuses:
 
-   - can't use for writing code to be turned into C extensions, only a wrapper tool.
+   - can't use for writing code to be turned into C extensions, only a wrapper
+     tool.
 
-5) SWIG (automatic wrapper generator)
+4) SWIG (automatic wrapper generator)
 
  - Plusses:
 
@@ -175,31 +173,32 @@ Thus:
  - Minuses:
 
    - generates lots of code between Python and the C code
-
-     - can cause performance problems that are nearly impossible to optimize out
-
+   - can cause performance problems that are nearly impossible to optimize
+     out
    - interface files can be hard to write
-   - doesn't necessarily avoid reference counting issues or needing to know API's
+   - doesn't necessarily avoid reference counting issues or needing to know
+     API's
 
-7) Weave
+5) scipy.weave
 
  - Plusses:
 
-   - Phenomenal tool
    - can turn many numpy expressions into C code
    - dynamic compiling and loading of generated C code
-   - can embed pure C code in Python module and have weave extract, generate interfaces
-     and compile, etc.
+   - can embed pure C code in Python module and have weave extract, generate
+     interfaces and compile, etc.
 
  - Minuses:
 
-   - Future uncertain--lacks a champion
+   - Future very uncertain: it's the only part of Scipy not ported to Python 3
+     and is effectively deprecated in favor of Cython.
 
-8) Psyco
+6) Psyco
 
  - Plusses:
 
-   - Turns pure python into efficient machine code through jit-like optimizations
+   - Turns pure python into efficient machine code through jit-like
+     optimizations
    - very fast when it optimizes well
 
  - Minuses:
@@ -209,15 +208,20 @@ Thus:
 
 Interfacing to Fortran:
 -----------------------
-Fortran: Clear choice is f2py. (Pyfort is an older alternative, but not supported
-any longer)
+The clear choice to wrap Fortran code is
+`f2py <https://docs.scipy.org/doc/numpy/f2py/>`_.
+
+Pyfort is an older alternative, but not supported any longer.
+Fwrap is a newer project that looked promising but isn't being developed any
+longer.
 
 Interfacing to C++:
 -------------------
-1) CXX
-2) Boost.python
-3) SWIG
-4) Sage has used cython to wrap C++ (not pretty, but it can be done)
-5) SIP (used mainly in PyQT)
+ 1) Cython
+ 2) CXX
+ 3) Boost.python
+ 4) SWIG
+ 5) SIP (used mainly in PyQT)
 
 """
+from __future__ import division, absolute_import, print_function
